@@ -1,16 +1,12 @@
-#!/usr/bin/env python3
-
 # Utility to parse and validate a HAT package
 
 import ctypes
-from typing import Any, List, Union
 from collections import OrderedDict
-from functools import partial
-
-from .hat_file import HATFile, Function, Parameter
-from .arg_info import ArgInfo, verify_args
-
 import os
+from typing import List
+
+from .hat_file import HATFile, Function
+from .function_info import FunctionInfo
 
 
 class HATPackage:
@@ -32,10 +28,13 @@ class HATPackage:
 
         self.functions = self.hat_file.functions
 
-    def get_functions(self):
+    def __iter__(self):
+        return iter(self.hat_file.functions)
+
+    def get_functions(self) -> List[Function]:
         return self.hat_file.functions
 
-    def get_functions_for_target(self, os: str, arch: str, required_extensions: list = []):
+    def get_functions_for_target(self, os: str, arch: str, required_extensions: list = []) -> List[Function]:
         all_functions = self.get_functions()
 
         def matches_target(hat_function):
@@ -49,6 +48,33 @@ class HATPackage:
 
         return list(filter(matches_target, all_functions))
 
+    def benchmark(
+        self,
+        functions: List[Function] = None,
+        store_in_hat=False,
+        batch_size=10,
+        min_time_in_sec=10,
+        input_sets_minimum_size_MB=50,
+        gpu_id: int = 0,
+        verbose: bool = False
+    ) -> List["hatlib.Result"]:
+        "Benchmarks the selected functions in the HAT package. If none are selected, all functions in the package are benchmarked."
+
+        functions = functions if functions is not None else self.functions
+
+        from benchmark_hat_package import run_benchmark
+
+        return run_benchmark(
+            self.hat_file_path,
+            store_in_hat=store_in_hat,
+            batch_size=batch_size,
+            min_time_in_sec=min_time_in_sec,
+            input_sets_minimum_size_MB=input_sets_minimum_size_MB,
+            gpu_id=gpu_id,
+            verbose=verbose,
+            functions=[f.name for f in functions]
+        )
+
 
 class AttributeDict(OrderedDict):
     """ Dictionary that allows entries to be accessed like attributes
@@ -66,21 +92,25 @@ class AttributeDict(OrderedDict):
         return OrderedDict.__getitem__(key)
 
 
-def _make_cpu_func(shared_lib: ctypes.CDLL, function_name: str, arg_infos: List[Parameter]):
-    arg_infos = [ArgInfo(d) for d in arg_infos]
-    fn = shared_lib[function_name]
+def _make_cpu_func(shared_lib: ctypes.CDLL, func: Function):
+    func_info = FunctionInfo(func)
+    fn = shared_lib[func_info.name]
 
     def f(*args):
-        # verify that the (numpy) input args match the description in
-        # the hat file
-        verify_args(args, arg_infos, function_name)
+        args_ = func_info.preprocess(args)
+
+        # verify that the args match the description in the hat file
+        func_info.verify(args_)
 
         # prepare the args to the hat package
-        hat_args = [arg.ctypes.data_as(arg_info.ctypes_pointer_type) for arg, arg_info in zip(args, arg_infos)]
+        hat_args = func_info.as_cargs(args_)
 
         # call the function in the hat package
         fn(*hat_args)
 
+        # get any results after post-processing
+        return func_info.postprocess(args_, args)
+
     return f
 
 
@@ -97,7 +127,7 @@ def _load_pkg_binary_module(hat_pkg: HATPackage):
     shared_lib = None
     if os.path.isfile(hat_pkg.link_target_path):
 
-        supported_extensions = [".dll", ".so"]
+        supported_extensions = [".dll", ".so", ".dylib"]
         _, extension = os.path.splitext(hat_pkg.link_target_path)
 
         if extension and extension not in supported_extensions:
@@ -149,7 +179,7 @@ def hat_package_to_func_dict(hat_pkg: HATPackage) -> AttributeDict:
         launches = func_desc.launches
         if not launches and shared_lib:
 
-            func_dict[func_name] = _make_cpu_func(shared_lib, func_desc.name, func_desc.arguments)
+            func_dict[func_name] = _make_cpu_func(shared_lib, func_desc)
         else:
             device_func = hat_pkg.hat_file.device_function_map.get(launches)
 
